@@ -72,6 +72,8 @@ Please note that the following id format is required:
   s[0-9]+ refers to the sentence number in the document
   t[0-9]+ refers to the term number in the sentence 
 
+The padding of zeros is optional. 
+
 =head3 --log DIR
 
 Directory in which the output files will be stored. Default: log
@@ -283,22 +285,20 @@ See create-icpropagation.pl for more information.
 =head3 --vectormatrix FILE
 
 This is the matrix file that contains the vector information to 
-use with the vector measure. This is required if you specify vector 
-with the --measure option. 
+use with the vector measure. 
 
-This file is generated by the vector-input.pl program. An example 
-of this file can be found in the samples/ directory and is called 
-matrix.
+If you do not want to use the default, this file is generated by 
+the vector-input.pl program. An example of this file can be found 
+in the samples/ directory and is called matrix.
 
 =head3 --vectorindex FILE
 
 This is the index file that contains the vector information to 
-use with the vector measure. This is required if you specify vector 
-with the --measure option.
+use with the vector measure. 
 
-This file is generated by the vector-input.pl program. An example 
-of this file can be found in the samples/ directory and is called 
-index.
+If you do not want to use the default, this file is generated by the 
+vector-input.pl program. An example of this file can be found in the 
+samples/ directory and is called index.
 
 =head3 --debugfile FILE
 
@@ -573,7 +573,7 @@ sub load_XML_Input {
     open(INPUT, $inputfile) || die "Could not open input file: $inputfile\n";
     
     #  get the instance and instance information
-    my $id = ""; my @ontext = ();
+    my $id = ""; my @ontext = (); my $flag = 0;
     while(<INPUT>) { 
 	chomp;
 	
@@ -582,14 +582,26 @@ sub load_XML_Input {
 	if($_=~/<\!DOCTYPE/)   { next; }
 	if($_=~/<corpus lang/) { next; }
 	if($_=~/<\/corpus>/)   { next; }
-
-	if($_=~/<text id=\"(.*?)\"/) { $id = $1; next; }
-	if($_=~/<\/text>/) { 
-	    $instancehash{$id} = [@context];
-	    $id = ""; @context = (); next;
+	
+	if($_=~/<MMOs>/) { 
+	    print STDERR "\nERROR. This file looks like it is in mm-xml format rather than aw-xml format.\n";
+	    print STDERR "Please check the format of your input file.\n\n";
+	    &minimalUsageNotes();
+	    exit;
 	}
 
-	
+	#  get the id
+	if($_=~/<text id=\"(.*?)\"/) { $id = $1; next; }
+
+	#  store the context
+	if($_=~/<\/text>/) { 
+	    $instancehash{$id} = [@context];
+	    $id = ""; @context = (); 
+	    next;
+	}
+
+	#  if candidate option is set make certain
+	#  that each instance has a candidate tag
 	if(defined $opt_candidates) { 
 	    if($_=~/<head id=/) { 
 		if(! ($_=~/candidates=\"(.*?)\"/)) { 
@@ -601,8 +613,33 @@ sub load_XML_Input {
 		}                  
 	    }
 	}            
+	
+	#  so what is left over is the terms - check to make 
+	#  certain the format is correct if tagged
+	if($_=~/\<head/) { 
+	    $flag = 1;
+	    $_=~/<head id=\"(d[0-9]+)\.(s[0-9]+)\.(t[0-9]+)\"/;
+	    my $d = $1; my $s = $2; my $t = $3;
+	    if($d eq "" || $s eq "" || $t eq "") { 
+		
+		    print STDERR "\nERROR. This instance does not look to be in the correct format.\n";
+		    print STDERR "Instance: $_\n";
+		    print STDERR "Please check the format of your input file.\n\n";
+		    &minimalUsageNotes();
+		    exit;
+	    }
+	}
+	
+	#  add the terms to the context
 	push @context, $_; 
     }
+
+    if($flag == 0) { 
+	print STDERR "\nERROR. This file looks like it is in plain format rather than aw-xml format.\n";
+	print STDERR "Please check the format of your input file.\n\n";
+	&minimalUsageNotes();
+	exit;
+    }
 }
 
 sub setKey {
@@ -838,27 +875,6 @@ sub checkOptions {
 	}   
     }
     
-    # make certain the db file is specified if the vector measure 
-    # is being used
-    if($opt_measure=~/vector/) {
-	if(! (defined $opt_vectorindex)) {
-	    print STDERR "The --vectorindex and --vectormatrix option must be\n";
-	    print STDERR "specified when using the vector measure. An example\n";
-	    print STDERR "of the matrix and index files can be seen in the \n";
-	    print STDERR "samples/ directory.\n\n";
-	    &minimalUsageNotes();
-	    exit;
-	}
-	if(! (defined $opt_vectormatrix)) {
-	    print STDERR "The --vectorindex and --vectormatrix option must be\n";
-	    print STDERR "specified when using the vector measure.An example\n";
-	    print STDERR "of the index and matrix files can be seen in the\n";
-	    print STDERR "samples/ directory. The vector-input.pl program can\n";
-	    print STDERR "generate these files given your specific text. \n\n";
-	    &minimalUsageNotes();
-	    exit;
-	}
-    }
      if(defined $opt_stem) { 
 	if(! ($opt_measure=~/vector|lesk/) ) {
 	    print STDERR "The --stem option is only available\n";
@@ -913,19 +929,6 @@ sub checkOptions {
 	}
     }    
     
-    #  make certain the propagation file is specified if the resnik, 
-    #  jcn, or lin measure is being used
-    if($opt_measure=~/(res|lin|jcn)/) {
-	if(! (defined $opt_icpropagation) and !(defined $opt_icfrequency) ) {
-	    print STDERR "The --icpropagation or --icfrequency option must be\n";
-	    print STDERR "specified when using the res, lin or jcn measures.\n";
-	    print STDERR "An example of the propagation file can be seen in\n";
-	    print STDERR "the samples/ directory.\n\n";
-	    &minimalUsageNotes();
-	    exit;
-	}
-    } 
-
     #  the --smooth option can only be used with the icfrequency options
     if(defined $opt_smooth) {
 	if(!defined $opt_icfrequency) {
@@ -1266,7 +1269,7 @@ sub showHelp() {
 #  function to output the version number
 ##############################################################################
 sub showVersion {
-    print '$Id: umls-allwords-senserelate.pl,v 1.7 2011/05/03 13:58:59 btmcinnes Exp $';
+    print '$Id: umls-allwords-senserelate.pl,v 1.10 2011/05/20 15:17:08 btmcinnes Exp $';
     print "\nCopyright (c) 2011, Ted Pedersen & Bridget McInnes\n";
 }
 
